package com.meshsoftware;

import com.meshsoftware.xml.MySAXParser;


//XSLT validation
import java.io.*;
import java.util.ResourceBundle;
import java.net.URL;
import java.net.URI;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


public class Add {


  public Add() {
  }

  public String add(String request) throws Exception {
//       System.out.println("Add Request ---->"+request);
       validateAgainstXSD(request);
       String response  = transformToResponse(request);
//       System.out.println("Add Response ---->"+response);
       return response;

  }

   private void validateAgainstXSD(String request) 
     throws Exception {
       System.out.println("Validate against XSD ---->");
      String className = this.getClass().getName();
      className = className.replace('.', '/');
      ClassLoader classLoader = this.getClass().getClassLoader();
      URL classFileURL = classLoader.getResource(className+".class");
      if(classFileURL == null)
	throw new Exception("Unable to determine the Root Folder");
      String rootFolder = classFileURL.toString();
      int endIndex = rootFolder.lastIndexOf(className);
      rootFolder = rootFolder.substring(0, endIndex);
      System.out.println("Root Folder = "+rootFolder);

      String xsdUrl = rootFolder+"xsds/AddRequest.xsd";
      xsdUrl = uriSpaceEncode(xsdUrl);
      System.out.println("Using XSD @ :"+xsdUrl);
      MySAXParser parser = new MySAXParser();
      parser.setXSD(xsdUrl);
     try {
       parser.parse(request, true);
     } catch (Exception e) {
       System.out.println(e.getMessage());
       System.out.println("Exception while validating with XSD.. Exiting");
       throw new Exception(e.getMessage());
     }
     
   }

   private String transformToResponse(String request) throws Exception {
       System.out.println("Transform to Internal XML  ---->");
     // Use the static TransformerFactory.newInstance() method to instantiate 
     // a TransformerFactory. The javax.xml.transform.TransformerFactory 
     // system property setting determines the actual class to instantiate --
     // org.apache.xalan.transformer.TransformerImpl.
     TransformerFactory tFactory = TransformerFactory.newInstance();
	
     // Use the TransformerFactory to instantiate a Transformer that will work
     // with  the stylesheet you specify. This method call also processes the 
     // stylesheet into a compiled Templates object.
     //System.out.println(System.getProperty("javax.xml.transform.TransformerFactory"));
     Transformer transformer = null;
     //XSD is stored in the classes folder of the webapps. 
     // Since it contains business logic we do not want it accessible 
     // as a URL like the XSD. Hence we need a mechanism to get to the XSL.
      String className = this.getClass().getName();
      className = className.replace('.', '/');
      ClassLoader classLoader = this.getClass().getClassLoader();
      URL classFileURL = classLoader.getResource(className+".class");
      if(classFileURL == null)
	throw new Exception("Unable to determine the Root Folder");
      String rootFolder = classFileURL.toString();
      int endIndex = rootFolder.lastIndexOf(className);
      rootFolder = rootFolder.substring(0, endIndex);
      String xslUrl = rootFolder+"xsls/Add.xsl";
      xslUrl = uriSpaceEncode(xslUrl);

      System.out.println("Using XSL @ "+xslUrl);

     try {
//	 transformer = tFactory.newTransformer(new StreamSource(new File(xslUrl)));
	 transformer = tFactory.newTransformer(new StreamSource(new File(new URI(xslUrl))));
     } catch (TransformerConfigurationException tce) {
	 throw new Exception("Unable to load the XSL: "+tce.getMessage());
     }
     
     // Use the Transformer to apply the associated Templates object to an 
     StringWriter resultWriter = new StringWriter();
     try {
	 transformer.transform(new StreamSource(new StringReader(request)), 
			       new StreamResult(resultWriter));
     } catch (TransformerException te) {
	 throw new Exception("Error while transforming: "+te.getMessage());
     }
     return resultWriter.toString();

   }

    private String xmlEncode(String in) { 
	String[] delim = { "&", "<", "\"" };
	String[] values = { "&amp;", "&lt;", "&quot;" };

	java.lang.StringBuffer dest = new java.lang.StringBuffer(in);
	for(int i=0; i< delim.length; i++) {
	    String src = dest.toString();
	    dest = new java.lang.StringBuffer();
	    java.util.StringTokenizer st = 
		new java.util.StringTokenizer(src, delim[i], true);
	    while(st.hasMoreTokens()) {
		String token = st.nextToken();
		//		System.out.println("=> "+token);
		if(token.equals(delim[i])) {
		    dest.append(values[i]);
		} else {
		    dest.append(token);
		}
	    }
	    //	    System.out.println("===="+i+"==>"+dest);
	}
	return dest.toString();
    }
	
    private String uriSpaceEncode(String in) { 
	String[] delim = { " "};
	String[] values = { "%20" };

	java.lang.StringBuffer dest = new java.lang.StringBuffer(in);
	for(int i=0; i< delim.length; i++) {
	    String src = dest.toString();
	    dest = new java.lang.StringBuffer();
	    java.util.StringTokenizer st = 
		new java.util.StringTokenizer(src, delim[i], true);
	    while(st.hasMoreTokens()) {
		String token = st.nextToken();
		//		System.out.println("=> "+token);
		if(token.equals(delim[i])) {
		    dest.append(values[i]);
		} else {
		    dest.append(token);
		}
	    }
	    //	    System.out.println("===="+i+"==>"+dest);
	}
	return dest.toString();
    }


  public static void main(String[] args) throws Exception {
      String s = null; //will store the input XML

	if (args.length < 1) {
            System.err.println("Usage: cmd filename");
            System.exit(1);
        }

        try {
  	    System.out.println("xml file : " + args[0]);
            s = new String(); 
	    String str = new String(); 
            BufferedReader buf = null;
            buf = new BufferedReader(new FileReader(args[0]));

            while ( (str = buf.readLine() ) != null )
                s = s + str;
            buf.close();

        } catch (Exception e) {
            e.printStackTrace();
	    System.exit(-1);
        }

    Add a = new Add();
    System.out.println(a.add(s));
  }
}
