package com.meshsoftware.xml;

import java.io.*;

import org.w3c.dom.Document;
import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;

import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.SerializerFactory;
import org.apache.xalan.templates.OutputProperties;
import org.apache.axis.utils.XMLUtils;
import javax.xml.transform.OutputKeys;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

// import javax.xml.parsers.SAXParser;
import org.apache.xerces.parsers.SAXParser;


/**
 * Class MySAXParser provides an XML document validated
 * against the personal.xsd schema using SAXParser.
 *
 * The parser class itself extends DefaultHandler for now but can be refactored to
 * to accept another "value object" that extends DefaultHandler and listens to the
 * parser events as it populates its data attributes.
 *
 */


public class MySAXParser extends DefaultHandler implements GenericParser  {

    StringBuffer bufdata = new StringBuffer();
    private String load = null;
    private String xsdUrl = null;

    protected static final String DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";
    protected static final int DEFAULT_REPETITION = 1;
    protected static final boolean DEFAULT_NAMESPACES = true;
    protected static final boolean DEFAULT_NAMESPACE_PREFIXES = false;
    protected static final boolean DEFAULT_VALIDATION = true;
    protected static final boolean DEFAULT_SCHEMA_VALIDATION = true;
    protected static final boolean DEFAULT_SCHEMA_FULL_CHECKING = false;
    protected static final boolean DEFAULT_DYNAMIC_VALIDATION = false;
    protected static final boolean DEFAULT_MEMORY_USAGE = false;
    protected static final boolean DEFAULT_TAGGINESS = false;


    public MySAXParser() {}

    public MySAXParser(String xsdUrl)  {
        setXSD(xsdUrl);
    }

    public void setXSD(String xsdUrl)  {
        this.xsdUrl = xsdUrl;
    }

    public void parse(String s, boolean validate) throws Exception {

        InputSource inp = new InputSource(((InputStream)new ByteArrayInputStream(s.getBytes())));

        if (validate) System.out.println("MySAXParser :: WITH validation");
        else System.out.println("MySAXParser :: WITHOUT validation");

	XMLReader parser = null;

	try {
		parser = XMLReaderFactory.createXMLReader(DEFAULT_PARSER_NAME);
	} catch (Exception e)  {
		e.printStackTrace();
	}
	
	// set parser features
	try {
            parser.setFeature(STANDALONE_DOCUMENT, true);
            String schemaLocation = null;
            if (validate) {
                schemaLocation = "http://localhost:8080/demo/AddRequest.xsd";
                parser.setProperty(EXTERNAL_SCHEMA_LOCATION, xsdUrl);
                parser.setFeature(VALIDATION_FEATURE_ID, validate);
                parser.setFeature(SCHEMA_VALIDATION_FEATURE_ID,  validate);
                System.out.println("MySAXParser :: schema : " + xsdUrl);
            }

        } catch (Exception e) { // to refine this to catch the exact feature / property 
            e.printStackTrace();
            throw new Exception(e.getMessage());
        }

        parser.setContentHandler(this);
        parser.setErrorHandler(this);


        // java.util.Properties p = OutputProperties.getDefaultMethodProperties("xml");
        // p.setProperty(OutputKeys.METHOD, "xml");
	// Serializer serializer = SerializerFactory.getSerializer(p);
	// DOMSerializer domSerializer = serializer.asDOMSerializer(doc);
	// serializer.setOutputStream(System.out);
	// parser.setContentHandler((serializer.asDOMSerializer(doc)).asContentHandler());

	try {
		long startTime = System.currentTimeMillis();
		parser.parse(inp);
		long endTime = System.currentTimeMillis();
		long tc = endTime - startTime;
		System.out.println("MySAXParser :: parser time : " + tc);
	} catch (Exception e) {
		System.err.println("parser error - " + e.getMessage());
		e.printStackTrace();
                throw new Exception(e.getMessage());
	}

    }


    public void error (SAXParseException e) throws SAXException {
      System.out.println("Error parsing the file: "+e.getMessage());
      throw new SAXException(e.getMessage());
   }

   public void warning (SAXParseException e) {
      System.out.println("Problem parsing the file: "+e.getMessage());
   }

   public void fatalError (SAXParseException e) throws SAXException  {
      System.out.println("Error parsing the file: "+e.getMessage());
      throw new SAXException(e.getMessage());
   }


    public void endElement(String uri, String name, String qName) { 

        // System.out.println("name : " + name);

        if ("PartyName".equalsIgnoreCase(name)) {
            // System.out.println("name : "+ name);
            load = bufdata.toString().trim();
            // System.out.println("MySAXParser :: load : " + load);
        }

        bufdata.delete(0, bufdata.length());
        
    }


    public void characters (char buf [], int offset, int len) throws SAXException {

        int bufLength = buf.length;
        bufdata.append(buf, offset, len);
        // System.out.println("----> bufdata : " + bufdata.toString());

    }

  
    public String getLoad() {
        System.out.println("MySAXParser.getLoad() : "+ load);
        return this.load;
    }



    public static void main(String[] args) {

	if (args.length < 1) {
            System.err.println("Usage:\n\tcmd xmlFile");
	    System.err.println("  or");
	    System.err.println("\tcmd xmlFile xsdFile");
            System.exit(1);
        }

        GenericParser parser = new MySAXParser();

        try {
  	    System.out.println("xml file : " + args[0]);
            String s = new String(); String str = new String(); 
            BufferedReader buf = null;
            buf = new BufferedReader(new FileReader(args[0]));

            while ( (str = buf.readLine() ) != null )
                s = s + str;
            buf.close();

	    if(args.length >= 2) {
	      parser.setXSD(args[1]);
	      parser.parse(s, true);
	    } else
	      parser.parse(s, false);
 
            System.out.println("MySAXParser :: load : " + parser.getLoad());

        } catch (Exception e) {
            e.printStackTrace();
        }

        System.exit(0);
    }
}
