package com.meshsoftware.xml;

import java.util.HashMap;


/**
 * Class ParserFactory 
 */


public class ParserFactory {

     // store a hashMap of parserType to parser class
     // return instance to selected parser class
     // usage : factory = ParserFactory.getInstance();
     //         factory.getParserForName(parserType);
     // valid parserTypes : DOMParserXML.class SAXParser.class, XPParser.class


    private ParserFactory() {};

//  IMP NOTE :  Need to correct the Singleton implementation here - getInstance must return the sane instance of ParserFactory and not do a new.

    public static ParserFactory getInstance() {
        return new ParserFactory();
    }


    public GenericParser getParserForName(String parserType) throws ClassNotFoundException, InstantiationException, IllegalAccessException {

        Class c = Class.forName(parserType);
        return (GenericParser) c.newInstance();

    }


    public static void main (String args[]) {

        try {
        ParserFactory factory = ParserFactory.getInstance();
        GenericParser parser = factory.getParserForName("com.meshsoftware.xml.MySAXParser");
        if ( parser instanceof MySAXParser)
            System.out.println("MySAXParser");
        if (parser != null)
            System.out.println("Parser not null");
        else 
            System.out.println("Parser null");
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}

